@@ -25,7 +25,9 @@ General information
 This is written as a Twisted application and intended to be run using the
 twistd command. To start the soledad server, run:
 
-    twistd -n web --wsgi=leap.soledad.server.application --port=X
+    twistd -n web \
+        --wsgi=leap.soledad.server.application.wsgi_application \
+        --port=X
 
 An initscript is included and will be installed system wide to make it
 feasible to start and stop the Soledad server service using a standard
@@ -49,9 +51,7 @@ synchronization is handled by the `leap.soledad.server.auth` module.
 Shared database
 ---------------
 
-Each user may store password-encrypted recovery data in the shared database,
-as well as obtain a lock on the shared database in order to prevent creation
-of multiple secrets in parallel.
+Each user may store password-encrypted recovery data in the shared database.
 
 Recovery documents are stored in the database without any information that
 may identify the user. In order to achieve this, the doc_id of recovery
@@ -77,49 +77,35 @@ This has some implications:
   * Because of the above, it is recommended that recovery documents expire
     (not implemented yet) to prevent excess storage.
 
-Lock documents, on the other hand, may be more thoroughly protected by the
-server. Their doc_id's are calculated from the SHARED_DB_LOCK_DOC_ID_PREFIX
-and the user's uid.
-
 The authorization for creating, updating, deleting and retrieving recovery
-and lock documents on the shared database is handled by
-`leap.soledad.server.auth` module.
+documents on the shared database is handled by `leap.soledad.server.auth`
+module.
 """
 
-import configparser
 import urlparse
 import sys
 
-from u1db.remote import http_app, utils
-
-# Keep OpenSSL's tsafe before importing Twisted submodules so we can put
-# it back if Twisted==12.0.0 messes with it.
-from OpenSSL import tsafe
-old_tsafe = tsafe
-
-from twisted import version
-if version.base() == "12.0.0":
-    # Put OpenSSL's tsafe back into place. This can probably be removed if we
-    # come to use Twisted>=12.3.0.
-    sys.modules['OpenSSL.tsafe'] = old_tsafe
-
-from leap.soledad.server.auth import SoledadTokenAuthMiddleware
-from leap.soledad.server.gzip_middleware import GzipMiddleware
-from leap.soledad.server.lock_resource import LockResource
-from leap.soledad.server.sync import (
-    SyncResource,
-    MAX_REQUEST_SIZE,
-    MAX_ENTRY_SIZE,
-)
-
+from leap.soledad.common.l2db.remote import http_app, utils
 from leap.soledad.common import SHARED_DB_NAME
-from leap.soledad.common.couch import CouchServerState
+
+from leap.soledad.server.sync import SyncResource
+from leap.soledad.server.sync import MAX_REQUEST_SIZE
+from leap.soledad.server.sync import MAX_ENTRY_SIZE
+
+from ._version import get_versions
+
+
+__all__ = [
+    'SoledadApp',
+    '__version__',
+]
 
 
 # ----------------------------------------------------------------------------
 # Soledad WSGI application
 # ----------------------------------------------------------------------------
 
+
 class SoledadApp(http_app.HTTPApp):
     """
     Soledad WSGI application
@@ -163,7 +149,6 @@ http_app.url_to_resource.register(http_app.DocsResource)
 http_app.url_to_resource.register(http_app.DocResource)
 
 # register Soledad's new or modified resources
-http_app.url_to_resource.register(LockResource)
 http_app.url_to_resource.register(SyncResource)
 
 
@@ -208,7 +193,8 @@ class HTTPInvocationByMethodWithBody(
             try:
                 content_length = int(self.environ['CONTENT_LENGTH'])
             except (ValueError, KeyError):
-                raise http_app.BadRequest
+                # raise http_app.BadRequest
+                content_length = self.max_request_size
             if content_length <= 0:
                 raise http_app.BadRequest
             if content_length > self.max_request_size:
@@ -235,23 +221,22 @@ class HTTPInvocationByMethodWithBody(
                     meth_put = self._lookup('%s_put' % method)
                     meth_end = self._lookup('%s_end' % method)
                     while True:
-                        line = body_getline()
-                        entry = line.strip()
+                        entry = body_getline().strip()
                         if entry == ']':  # end of incoming document stream
                             break
                         if not entry or not comma:  # empty or no prec comma
                             raise http_app.BadRequest
                         entry, comma = utils.check_and_strip_comma(entry)
-                        meth_put({}, entry)
+                        content = body_getline().strip()
+                        content, comma = utils.check_and_strip_comma(content)
+                        meth_put({'content': content or None}, entry)
                     if comma or body_getline():  # extra comma or data
                         raise http_app.BadRequest
                     return meth_end()
                 # handle outgoing documents
                 elif content_type == 'application/x-soledad-sync-get':
-                    line = body_getline()
-                    entry = line.strip()
                     meth_get = self._lookup('%s_get' % method)
-                    return meth_get({}, line)
+                    return meth_get()
                 else:
                     raise http_app.BadRequest()
             else:
@@ -262,51 +247,5 @@ class HTTPInvocationByMethodWithBody(
 http_app.HTTPInvocationByMethodWithBody = HTTPInvocationByMethodWithBody
 
 
-# ----------------------------------------------------------------------------
-# Auxiliary functions
-# ----------------------------------------------------------------------------
-
-def load_configuration(file_path):
-    """
-    Load server configuration from file.
-
-    @param file_path: The path to the configuration file.
-    @type file_path: str
-
-    @return: A dictionary with the configuration.
-    @rtype: dict
-    """
-    conf = {
-        'couch_url': 'http://localhost:5984',
-    }
-    config = configparser.ConfigParser()
-    config.read(file_path)
-    if 'soledad-server' in config:
-        for key in conf:
-            if key in config['soledad-server']:
-                conf[key] = config['soledad-server'][key]
-    # TODO: implement basic parsing/sanitization of options comming from
-    # config file.
-    return conf
-
-
-# ----------------------------------------------------------------------------
-# Run as Twisted WSGI Resource
-# ----------------------------------------------------------------------------
-
-def application(environ, start_response):
-    conf = load_configuration('/etc/leap/soledad-server.conf')
-    state = CouchServerState(
-        conf['couch_url'],
-        SoledadApp.SHARED_DB_NAME,
-        SoledadTokenAuthMiddleware.TOKENS_DB)
-    # WSGI application that may be used by `twistd -web`
-    application = GzipMiddleware(
-        SoledadTokenAuthMiddleware(SoledadApp(state)))
-
-    return application(environ, start_response)
-
-
-from ._version import get_versions
 __version__ = get_versions()['version']
 del get_versions
